/*
 * OpenI2CRADIO
 * INternal eeprom utilities
 * Copyright (C) 2013-09-11 K.Ohta <whatisthis.sowhat ai gmail.com>
 * License: GPL2+LE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2,
 *  or (at your option) any later version.
 *  This library / program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this library; see the file COPYING. If not, write to the
 *  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  As a special exception, if you link this(includeed from sdcc) library
 *  with other files, some of which are compiled with SDCC,
 *  to produce an executable, this library does not by itself cause
 *  the resulting executable to be covered by the GNU General Public License.
 *  This exception does not however invalidate any other reasons why
 *  the executable file might be covered by the GNU General Public License.
 */

#ifndef EEPROM_UTIL_H
#define	EEPROM_UTIL_H

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if defined(__SDCC)
#include <sdcc-lib.h>
#include <pic18fregs.h> /* ONLY FOR PIC18x */
#include <delay.h>
#else
#include <xc.h>
#endif


#include "commondef.h"
#include "iodef.h"
#include "idle.h"
#include "i2c_io.h"
#include "eeprom.h"
#include "ioports.h"

#ifdef	__cplusplus
extern "C" {
#endif

    extern void save_eeprom(void);
    extern unsigned char load_eeprom(void);
    extern void check_eeprom(void);
/*
 * Frequency utils
 */
   extern void format_frequencies(unsigned int page);
   extern void check_frequencies(void);
   extern void save_frequencies(void);
   extern char save_frequencies_page(unsigned char page);
   extern int load_frequencies(unsigned int page, unsigned char check_only);
   extern int load_userbands(void);
   extern void save_userbands(void);
   extern void format_userbands(void);   
/*
 * Constants
 */
#define EEPROM_INTSET_MAGIC 0x1298
#define EEPROM_FREQSET_MAGIC 0xfabc
#define EEPROM_BANDSET_MAGIC 0xfd0f

#define EEPROM_SET_ADDR 0x0000
#define EEPROM_BANDSET_ADDR 0x0100
#define EEPROM_FREQSET_ADDR 0x0200   

#ifdef	__cplusplus
}
#endif

#endif	/* EEPROM_UTIL_H */

